## ---- include = FALSE------------------------------------------------------
suppressMessages({
    BiocInstaller::biocLite("waldronlab/TCGAutils")
})
suppressPackageStartupMessages({
    library(RTCGAToolbox)
    library(TCGAutils)
})

## ---- eval=FALSE-----------------------------------------------------------
#  library(RTCGAToolbox)
#  ## Download and install GitHub resource
#  if (!require(TCGAutils, quietly = TRUE)) {
#      BiocInstaller::biocLite("waldronlab/TCGAutils", suppressAutoUpdate = TRUE,
#          suppressUpdates = TRUE)
#  }
#  library(TCGAutils)

## --------------------------------------------------------------------------
# BIOCONDUCTOR
library(MultiAssayExperiment)
library(RaggedExperiment)
# CRAN
library(readr)

## --------------------------------------------------------------------------
dataset <- getFirehoseDatasets()[27] # PRAD
stopifnot(identical(dataset, "PRAD"))
runDate <- getFirehoseRunningDates(last = 1)
analyzeDate <- getFirehoseAnalyzeDates(last = 1)

## --------------------------------------------------------------------------
buildMultiAssayExperiment <- function(TCGAcode, runDate, analyzeDate, dataType=
    c("RNASeqGene", "RNASeq2GeneNorm", "miRNASeqGene", "CNASNP", "CNVSNP",
    "CNASeq", "CNACGH", "Methylation", "mRNAArray", "miRNAArray", "RPPAArray",
    "Mutation", "GISTIC"), dataDirectory = "/tmp", force = FALSE) {
        if (!dir.exists(dataDirectory))
            dir.create(dataDirectory)
    
    message("\n######\n", "\nProcessing ", TCGAcode, " : )\n", "\n######\n")
    serialPath <- file.path(dataDirectory, paste0(TCGAcode, ".rds"))
    
    targets <- c("RNASeqGene", "RNASeq2GeneNorm", "miRNASeqGene",
                     "CNASNP", "CNVSNP", "CNASeq", "CNACGH", "Methylation",
                     "mRNAArray", "miRNAArray", "RPPAArray", "Mutation",
                     "GISTIC")
    names(targets) <- targets
    
    dataType <- match.arg(dataType, targets, several.ok = TRUE)
    names(dataType) <- dataType
    dataType <- rep(TRUE, length(dataType))
    
    if (file.exists(serialPath)) {
        cancer.object <- readRDS(serialPath)
    } else {
        cancer.object <- do.call(getFirehoseData,
            args = c(list(dataset = TCGAcode,
                runDate = runDate,
                gistic2Date = analyzeDate,
                RNAseqNorm = "raw_counts",
                RNAseq2Norm =
                    "normalized_count",
                forceDownload = FALSE,
                destdir = dataDirectory,
                fileSizeLimit = 500000,
                getUUIDs = FALSE), as.list(dataType))
            )
        # Uncomment to save to your directory
        # saveRDS(cancer.object, file = serialPath, compress = "bzip2")
    }
    ## Add clinical data from RTCGAToolbox
    clinical.data <- selectType(cancer.object, "clinical")
    rownames(clinical.data) <- toupper(gsub("\\.", "-",
                                            rownames(clinical.data)))
    clinical.data[] <- apply(clinical.data, 2, type.convert)
    
    dataList <- lapply(targets, function(datType) {
        tryCatch({biocExtract(cancer.object, datType)},
                 error = function(e) {
                     message(datType, " does not contain any data!")
                 })
    })
    data.full <- Filter(length, dataList)
    
    isList <- vapply(data.full, is.list, logical(1L))
    unlisted <- unlist(data.full[isList])
    data.full <- c(data.full[!isList], unlisted)
    newmap <- TCGAutils::generateMap(data.full, clinical.data,
        TCGAutils::TCGAbarcode)
    mae <- MultiAssayExperiment(data.full, clinical.data, newmap)
    return( mae )
}

## --------------------------------------------------------------------------
buildMultiAssayExperiment(TCGAcode=dataset, runDate=runDate,
    analyzeDate=analyzeDate, dataType = "miRNASeqGene", dataDirectory = "tmp")

